# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
#
# SPDX-License-Identifier: Apache-2.0
#
# Portions derived from  https://github.com/microsoft/autogen are under the MIT License.
# SPDX-License-Identifier: MIT
# !/usr/bin/env python3 -m pytest

import pytest

from autogen.import_utils import run_for_optional_imports
from autogen.token_count_utils import (
    _num_token_from_messages,
    count_token,
    get_max_token_limit,
    num_tokens_from_functions,
    percentile_used,
    token_left,
)

func1 = {
    "name": "sh",
    "description": "run a shell script and return the execution result.",
    "parameters": {
        "type": "object",
        "properties": {
            "script": {
                "type": "string",
                "description": "Valid shell script to execute.",
            }
        },
        "required": ["script"],
    },
}
func2 = {
    "name": "query_wolfram",
    "description": "Return the API query result from the Wolfram Alpha. the return is a tuple of (result, is_success).",
    "parameters": {
        "type": "object",
        "properties": {},
    },
}
func3 = {
    "name": "python",
    "description": "run cell in ipython and return the execution result.",
    "parameters": {
        "type": "object",
        "properties": {
            "cell": {
                "type": "string",
                "description": "Valid Python cell to execute.",
            }
        },
        "required": ["cell"],
    },
}


@pytest.mark.parametrize(
    "input_functions, expected_count", [([func1], 44), ([func2], 46), ([func3], 45), ([func1, func2], 78)]
)
def test_num_tokens_from_functions(input_functions, expected_count):
    assert num_tokens_from_functions(input_functions) == expected_count


@pytest.mark.parametrize(
    "model, expected_count",
    [
        ("mistral-", 524),
        ("deepseek-chat", 524),
        ("claude", 524),
        ("gemini", 524),
    ],
)
def test_num_token_from_messages(model: str, expected_count: int) -> None:
    messages = [
        {
            "content": "You are a helpful AI assistant.\nSolve tasks using your coding and language skills.\nIn the following cases, suggest python code (in a python coding block) or shell script (in a sh coding block) for the user to execute.\n    1. When you need to collect info, use the code to output the info you need, for example, browse or search the web, download/read a file, print the content of a webpage or a file, get the current date/time, check the operating system. After sufficient info is printed and the task is ready to be solved based on your language skill, you can solve the task by yourself.\n    2. When you need to perform some task with code, use the code to perform the task and output the result. Finish the task smartly.\nSolve the task step by step if you need to. If a plan is not provided, explain your plan first. Be clear which step uses code, and which step uses your language skill.\nWhen using code, you must indicate the script type in the code block. The user cannot provide any other feedback or perform any other action beyond executing the code you suggest. The user can't modify your code. So do not suggest incomplete code which requires users to modify. Don't use a code block if it's not intended to be executed by the user.\nIf you want the user to save the code in a file before executing it, put # filename: <filename> inside the code block as the first line. Don't include multiple code blocks in one response. Do not ask users to copy and paste the result. Instead, use 'print' function for the output when relevant. Check the execution result returned by the user.\nIf the result indicates there is an error, fix the error and output the code again. Suggest the full code instead of partial code or code changes. If the error can't be fixed or if the task is not solved even after the code is executed successfully, analyze the problem, revisit your assumption, collect additional info you need, and think of a different approach to try.\nWhen you find an answer, verify the answer carefully. Include verifiable evidence in your response if possible.\nReply \"TERMINATE\" in the end when everything is done.\n    ",
            "role": "system",
        },
        {"content": "Hi", "role": "user", "name": "user_proxy"},
        {
            "content": "Hello! How can I assist you today? If you have a specific task or question, feel free to share it, and I'll do my best to help.",
            "role": "assistant",
            "name": "assistant",
        },
        {"content": "okkk", "role": "user", "name": "user_proxy"},
    ]
    assert _num_token_from_messages(messages=messages, model=model) == expected_count


@run_for_optional_imports("PIL", "unknown")
def test_num_tokens_from_gpt_image():
    # mock num_tokens_from_gpt_image function
    base64_encoded_image = (
        "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4"
        "//8/w38GIAXDIBKE0DHxgljNBAAO9TXL0Y4OHwAAAABJRU5ErkJggg=="
    )

    messages = [
        {
            "role": "system",
            "content": "you are a helpful assistant. af3758 *3 33(3)",
        },
        {
            "role": "user",
            "content": [
                {"type": "text", "text": "hello asdfjj qeweee"},
                {"type": "image_url", "image_url": {"url": base64_encoded_image}},
            ],
        },
    ]
    tokens = count_token(messages, model="gpt-4-vision-preview")

    # The total number of tokens is text + image
    # where text = 34, as shown in the previous test case
    # the image token is: 85 + 170 = 255
    assert tokens == 34 + 255

    # Test low quality
    messages = [
        {
            "role": "system",
            "content": "you are a helpful assistant. af3758 *3 33(3)",
        },
        {
            "role": "user",
            "content": [
                {"type": "text", "text": "hello asdfjj qeweee"},
                {"type": "image_url", "image_url": {"url": base64_encoded_image, "detail": "low"}},
            ],
        },
    ]
    tokens = count_token(messages, model="gpt-4o")
    assert tokens == 34 + 85


def test_count_token():
    messages = [
        {
            "role": "system",
            "content": "you are a helpful assistant. af3758 *3 33(3)",
        },
        {
            "role": "user",
            "content": "hello asdfjj qeweee",
        },
    ]
    assert count_token(messages) == 34
    assert percentile_used(messages) == 34 / 4096
    assert token_left(messages) == 4096 - 34

    text = "I'm sorry, but I'm not able to"
    assert count_token(text) == 10
    assert token_left(text) == 4096 - 10
    assert percentile_used(text) == 10 / 4096


def test_model_aliases():
    assert get_max_token_limit("gpt35-turbo") == get_max_token_limit("gpt-3.5-turbo")
    assert get_max_token_limit("gpt-35-turbo") == get_max_token_limit("gpt-3.5-turbo")
    assert get_max_token_limit("gpt4") == get_max_token_limit("gpt-4")
    assert get_max_token_limit("gpt4-32k") == get_max_token_limit("gpt-4-32k")
    assert get_max_token_limit("gpt4o") == get_max_token_limit("gpt-4o")
    assert get_max_token_limit("gpt4o-mini") == get_max_token_limit("gpt-4o-mini")


if __name__ == "__main__":
    #    test_num_tokens_from_functions()
    #    test_count_token()
    test_model_aliases()



# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
#
# SPDX-License-Identifier: Apache-2.0

import pytest
from jsonschema.exceptions import _RefResolutionError

from autogen.json_utils import resolve_json_references  # Replace 'your_module' with the actual module name


def test_resolve_json_references_no_refs() -> None:
    schema = {"type": "object", "properties": {"name": {"type": "string"}, "age": {"type": "integer"}}}
    resolved_schema = resolve_json_references(schema)
    assert resolved_schema == schema


def test_resolve_json_references_with_refs() -> None:
    schema = {
        "type": "object",
        "properties": {"name": {"type": "string"}, "address": {"$ref": "#/definitions/address"}},
        "definitions": {
            "address": {"type": "object", "properties": {"street": {"type": "string"}, "city": {"type": "string"}}}
        },
    }
    expected_resolved_schema = {
        "type": "object",
        "properties": {
            "name": {"type": "string"},
            "address": {"type": "object", "properties": {"street": {"type": "string"}, "city": {"type": "string"}}},
        },
        "definitions": {
            "address": {"type": "object", "properties": {"street": {"type": "string"}, "city": {"type": "string"}}}
        },
    }
    resolved_schema = resolve_json_references(schema)
    assert resolved_schema == expected_resolved_schema


def test_resolve_json_references_invalid_ref() -> None:
    schema = {
        "type": "object",
        "properties": {"name": {"type": "string"}, "address": {"$ref": "#/definitions/non_existent"}},
        "definitions": {
            "address": {"type": "object", "properties": {"street": {"type": "string"}, "city": {"type": "string"}}}
        },
    }
    with pytest.raises(_RefResolutionError, match="Unresolvable JSON pointer: 'definitions/non_existent'"):
        resolve_json_references(schema)



# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
#
# SPDX-License-Identifier: Apache-2.0
#
# Portions derived from  https://github.com/microsoft/autogen are under the MIT License.
# SPDX-License-Identifier: MIT
# !/usr/bin/env python3 -m pytest

import os
import tempfile
from io import StringIO
from types import SimpleNamespace
from unittest.mock import patch

import pytest

from autogen.code_utils import (
    UNKNOWN,
    check_can_use_docker_or_throw,
    content_str,
    create_virtual_env,
    decide_use_docker,
    execute_code,
    extract_code,
    get_powershell_command,
    in_docker_container,
    infer_lang,
    is_docker_running,
)

here = os.path.abspath(os.path.dirname(__file__))

skip_docker_test = not (is_docker_running() and decide_use_docker(use_docker=None))


def test_infer_lang():
    assert infer_lang("print('hello world')") == "python"
    assert infer_lang("pip install autogen") == "sh"

    # test infer lang for unknown code/invalid code
    assert infer_lang("dummy text") == UNKNOWN
    assert infer_lang("print('hello world'))") == UNKNOWN


def test_extract_code():
    print(extract_code("```bash\npython temp.py\n```"))
    # test extract_code from markdown
    codeblocks = extract_code(
        """
Example:
```
print("hello extract code")
```
""",
        detect_single_line_code=False,
    )
    print(codeblocks)

    codeblocks2 = extract_code(
        """
Example:
```
print("hello extract code")
```
""",
        detect_single_line_code=True,
    )
    print(codeblocks2)

    assert codeblocks2 == codeblocks
    # import pdb; pdb.set_trace()

    codeblocks = extract_code(
        """
Example:
```python
def scrape(url):
    import requests
    from bs4 import BeautifulSoup
    response = requests.get(url)
    soup = BeautifulSoup(response.text, "html.parser")
    title = soup.find("title").text
    text = soup.find("div", {"id": "bodyContent"}).text
    return title, text
```
Test:
```python
url = "https://en.wikipedia.org/wiki/Web_scraping"
title, text = scrape(url)
print(f"Title: {title}")
print(f"Text: {text}")
```
"""
    )
    print(codeblocks)
    assert len(codeblocks) == 2 and codeblocks[0][0] == "python" and codeblocks[1][0] == "python"

    codeblocks = extract_code(
        """
Example:
```python
def scrape(url):
    import requests
    from bs4 import BeautifulSoup
    response = requests.get(url)
    soup = BeautifulSoup(response.text, "html.parser")
    title = soup.find("title").text
    text = soup.find("div", {"id": "bodyContent"}).text
    return title, text
```
Test:
```python
url = "https://en.wikipedia.org/wiki/Web_scraping"
title, text = scrape(url)
print(f"Title: {title}")
print(f"Text: {text}")
```
"""
    )
    print(codeblocks)
    assert len(codeblocks) == 2 and codeblocks[0][0] == "python" and codeblocks[1][0] == "python"

    # Check for indented code blocks
    codeblocks = extract_code(
        """
Example:
   ```python
   def scrape(url):
       import requests
       from bs4 import BeautifulSoup
       response = requests.get(url)
       soup = BeautifulSoup(response.text, "html.parser")
       title = soup.find("title").text
       text = soup.find("div", {"id": "bodyContent"}).text
       return title, text
   ```
"""
    )
    print(codeblocks)
    assert len(codeblocks) == 1 and codeblocks[0][0] == "python"

    # Check for codeblocks with \r\n
    codeblocks = extract_code(
        """
Example:
```python
def scrape(url):
   import requests
   from bs4 import BeautifulSoup
   response = requests.get(url)
   soup = BeautifulSoup(response.text, "html.parser")
   title = soup.find("title").text
   text = soup.find("div", {"id": "bodyContent"}).text
   return title, text
```
""".replace("\n", "\r\n")
    )
    print(codeblocks)
    assert len(codeblocks) == 1 and codeblocks[0][0] == "python"

    codeblocks = extract_code("no code block")
    assert len(codeblocks) == 1 and codeblocks[0] == (UNKNOWN, "no code block")

    # Disable single line code detection
    line = "Run `source setup.sh` from terminal"
    codeblocks = extract_code(line, detect_single_line_code=False)
    assert len(codeblocks) == 1 and codeblocks[0] == (UNKNOWN, line)

    # Enable single line code detection
    codeblocks = extract_code("Run `source setup.sh` from terminal", detect_single_line_code=True)
    assert len(codeblocks) == 1 and codeblocks[0] == ("", "source setup.sh")


@pytest.mark.docker
@pytest.mark.skipif(skip_docker_test, reason="docker is not running or requested to skip docker tests")
def test_execute_code(use_docker=True):
    # Test execute code and save the code to a file.
    with tempfile.TemporaryDirectory() as tempdir:
        filename = "temp_file_with_code.py"

        # execute code and save the code to a file.
        exit_code, msg, image = execute_code(
            "print('hello world')",
            filename=filename,
            work_dir=tempdir,
            use_docker=use_docker,
        )
        assert exit_code == 0 and msg == "hello world\n", msg

        # read the file just saved
        exit_code, msg, image = execute_code(
            f"with open('{filename}', 'rt') as f: print(f.read())",
            use_docker=use_docker,
            work_dir=tempdir,
        )
        assert exit_code == 0 and "print('hello world')" in msg, msg

        # execute code in a file
        exit_code, msg, image = execute_code(
            filename=filename,
            use_docker=use_docker,
            work_dir=tempdir,
        )
        assert exit_code == 0 and msg == "hello world\n", msg

        # execute code in a file using shell command directly
        exit_code, msg, image = execute_code(
            f"python {filename}",
            lang="sh",
            use_docker=use_docker,
            work_dir=tempdir,
        )
        assert exit_code == 0 and msg == "hello world\n", msg

    with tempfile.TemporaryDirectory() as tempdir:
        # execute code for assertion error
        exit_code, msg, image = execute_code(
            "assert 1==2",
            use_docker=use_docker,
            work_dir=tempdir,
        )
        assert exit_code, msg
        assert "AssertionError" in msg
        assert 'File "' in msg or 'File ".\\"' in msg  # py3.8 + win32

    with tempfile.TemporaryDirectory() as tempdir:
        # execute code which takes a long time
        exit_code, error, image = execute_code(
            "import time; time.sleep(2)",
            timeout=1,
            use_docker=use_docker,
            work_dir=tempdir,
        )
        assert exit_code and error == "Timeout"
        if use_docker is True:
            assert isinstance(image, str)


@pytest.mark.docker
@pytest.mark.skipif(skip_docker_test, reason="docker is not running or requested to skip docker tests")
def test_execute_code_with_custom_filename_on_docker():
    with tempfile.TemporaryDirectory() as tempdir:
        filename = "codetest.py"
        exit_code, msg, image = execute_code(
            "print('hello world')",
            filename=filename,
            use_docker=True,
            work_dir=tempdir,
        )
        assert exit_code == 0 and msg == "hello world\n", msg
        assert image == "python:codetest.py"


@pytest.mark.docker
@pytest.mark.skipif(
    skip_docker_test,
    reason="docker is not running or requested to skip docker tests",
)
def test_execute_code_with_misformed_filename_on_docker():
    with tempfile.TemporaryDirectory() as tempdir:
        filename = "codetest.py (some extra information)"
        exit_code, msg, image = execute_code(
            "print('hello world')",
            filename=filename,
            use_docker=True,
            work_dir=tempdir,
        )
        assert exit_code == 0 and msg == "hello world\n", msg
        assert image == "python:codetest.py__some_extra_information_"


def test_execute_code_raises_when_code_and_filename_are_both_none():
    with pytest.raises(AssertionError):
        execute_code(code=None, filename=None)


def test_execute_code_no_docker():
    test_execute_code(use_docker=False)


def test_execute_code_timeout_no_docker():
    exit_code, error, image = execute_code("import time; time.sleep(2)", timeout=1, use_docker=False)
    assert exit_code and error == "Timeout"
    assert image is None


def get_current_autogen_env_var():
    return os.environ.get("AUTOGEN_USE_DOCKER", None)


def restore_autogen_env_var(current_env_value):
    if current_env_value is None:
        del os.environ["AUTOGEN_USE_DOCKER"]
    else:
        os.environ["AUTOGEN_USE_DOCKER"] = current_env_value


def test_decide_use_docker_truthy_values():
    current_env_value = get_current_autogen_env_var()

    for truthy_value in ["1", "true", "yes", "t"]:
        os.environ["AUTOGEN_USE_DOCKER"] = truthy_value
        assert decide_use_docker(None) is True

    restore_autogen_env_var(current_env_value)


def test_decide_use_docker_falsy_values():
    current_env_value = get_current_autogen_env_var()

    for falsy_value in ["0", "false", "no", "f"]:
        os.environ["AUTOGEN_USE_DOCKER"] = falsy_value
        assert decide_use_docker(None) is False

    restore_autogen_env_var(current_env_value)


def test_decide_use_docker():
    current_env_value = get_current_autogen_env_var()

    os.environ["AUTOGEN_USE_DOCKER"] = "none"
    assert decide_use_docker(None) is None
    os.environ["AUTOGEN_USE_DOCKER"] = "invalid"
    with pytest.raises(ValueError):
        decide_use_docker(None)

    restore_autogen_env_var(current_env_value)


def test_decide_use_docker_with_env_var():
    current_env_value = get_current_autogen_env_var()

    os.environ["AUTOGEN_USE_DOCKER"] = "false"
    assert decide_use_docker(None) is False
    os.environ["AUTOGEN_USE_DOCKER"] = "true"
    assert decide_use_docker(None) is True
    os.environ["AUTOGEN_USE_DOCKER"] = "none"
    assert decide_use_docker(None) is None
    os.environ["AUTOGEN_USE_DOCKER"] = "invalid"
    with pytest.raises(ValueError):
        decide_use_docker(None)

    restore_autogen_env_var(current_env_value)


def test_decide_use_docker_with_env_var_and_argument():
    current_env_value = get_current_autogen_env_var()

    os.environ["AUTOGEN_USE_DOCKER"] = "false"
    assert decide_use_docker(True) is True
    os.environ["AUTOGEN_USE_DOCKER"] = "true"
    assert decide_use_docker(False) is False
    os.environ["AUTOGEN_USE_DOCKER"] = "none"
    assert decide_use_docker(True) is True
    os.environ["AUTOGEN_USE_DOCKER"] = "invalid"
    assert decide_use_docker(True) is True

    restore_autogen_env_var(current_env_value)


def test_can_use_docker_or_throw():
    check_can_use_docker_or_throw(None)
    if not is_docker_running() and not in_docker_container():
        check_can_use_docker_or_throw(False)
    if not is_docker_running() and not in_docker_container():
        with pytest.raises(RuntimeError):
            check_can_use_docker_or_throw(True)


def test_create_virtual_env():
    with tempfile.TemporaryDirectory() as temp_dir:
        venv_context = create_virtual_env(temp_dir)
        assert isinstance(venv_context, SimpleNamespace)
        assert venv_context.env_name == os.path.split(temp_dir)[1]


def test_create_virtual_env_with_extra_args():
    with tempfile.TemporaryDirectory() as temp_dir:
        venv_context = create_virtual_env(temp_dir, with_pip=False)
        assert isinstance(venv_context, SimpleNamespace)
        assert venv_context.env_name == os.path.split(temp_dir)[1]


class TestContentStr:
    def test_string_content(self):
        assert content_str("simple string") == "simple string"

    def test_list_of_text_content(self):
        content = [{"type": "text", "text": "hello"}, {"type": "text", "text": " world"}]
        assert content_str(content) == "hello world"

    def test_mixed_content(self):
        content = [{"type": "text", "text": "hello"}, {"type": "image_url", "url": "http://example.com/image.png"}]
        assert content_str(content) == "hello<image>"

    def test_invalid_content(self):
        content = [{"type": "text", "text": "hello"}, {"type": "wrong_type", "url": "http://example.com/image.png"}]
        with pytest.raises(ValueError):
            content_str(content)

    def test_empty_list(self):
        assert content_str([]) == ""

    def test_non_dict_in_list(self):
        content = ["string", {"type": "text", "text": "text"}]
        with pytest.raises(TypeError):
            content_str(content)


class TestGetPowerShellCommand:
    @patch("subprocess.run")
    def test_get_powershell_command_powershell(self, mock_subprocess_run):
        # Set up the mock to return a successful result for 'powershell'
        mock_subprocess_run.return_value.returncode = 0
        mock_subprocess_run.return_value.stdout = StringIO("5")

        assert get_powershell_command() == "powershell"

    @patch("subprocess.run")
    def test_get_powershell_command_pwsh(self, mock_subprocess_run):
        # Set up the mock to return a successful result for 'pwsh'
        mock_subprocess_run.side_effect = [FileNotFoundError, mock_subprocess_run.return_value]
        mock_subprocess_run.return_value.returncode = 0
        mock_subprocess_run.return_value.stdout = StringIO("7")

        assert get_powershell_command() == "pwsh"

    @patch("subprocess.run")
    def test_get_powershell_command_not_found(self, mock_subprocess_run):
        mock_subprocess_run.side_effect = [FileNotFoundError, FileNotFoundError]
        with pytest.raises(FileNotFoundError):
            get_powershell_command()

    @patch("subprocess.run")
    def test_get_powershell_command_no_permission(self, mock_subprocess_run):
        mock_subprocess_run.side_effect = [PermissionError, FileNotFoundError]
        with pytest.raises(PermissionError):
            get_powershell_command()


if __name__ == "__main__":
    # test_infer_lang()
    test_extract_code()
    # test_execute_code()
    # test_find_code()
    # unittest.main()



# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
#
# SPDX-License-Identifier: Apache-2.0

import json
import tempfile
from _collections_abc import dict_items, dict_keys, dict_values
from copy import copy, deepcopy
from typing import Any

import pytest

from autogen.llm_config import LLMConfig
from autogen.oai.anthropic import AnthropicLLMConfigEntry
from autogen.oai.bedrock import BedrockLLMConfigEntry
from autogen.oai.cerebras import CerebrasLLMConfigEntry
from autogen.oai.client import AzureOpenAILLMConfigEntry, DeepSeekLLMConfigEntry, OpenAILLMConfigEntry
from autogen.oai.cohere import CohereLLMConfigEntry
from autogen.oai.gemini import GeminiLLMConfigEntry
from autogen.oai.groq import GroqLLMConfigEntry
from autogen.oai.mistral import MistralLLMConfigEntry
from autogen.oai.ollama import OllamaLLMConfigEntry
from autogen.oai.together import TogetherLLMConfigEntry

JSON_SAMPLE = """
[
    {
        "model": "gpt-3.5-turbo",
        "api_type": "openai",
        "tags": ["gpt35"]
    },
    {
        "model": "gpt-4",
        "api_type": "openai",
        "tags": ["gpt4"]
    },
    {
        "model": "gpt-35-turbo-v0301",
        "tags": ["gpt-3.5-turbo", "gpt35_turbo"],
        "api_key": "Your Azure OAI API Key",
        "base_url": "https://deployment_name.openai.azure.com/",
        "api_type": "azure",
        "api_version": "2024-02-01"
    },
    {
        "model": "gpt",
        "api_key": "not-needed",
        "base_url": "http://localhost:1234/v1",
        "tags": []
    }
]
"""

JSON_SAMPLE_DICT = json.loads(JSON_SAMPLE)


@pytest.fixture
def openai_llm_config_entry() -> OpenAILLMConfigEntry:
    return OpenAILLMConfigEntry(model="gpt-4o-mini", api_key="sk-mockopenaiAPIkeysinexpectedformatsfortestingonly")


class TestLLMConfigEntry:
    def test_extra_fields(self) -> None:
        with pytest.raises(ValueError) as e:
            # Intentionally passing extra field to raise an error
            OpenAILLMConfigEntry(  # type: ignore [call-arg]
                model="gpt-4o-mini", api_key="sk-mockopenaiAPIkeysinexpectedformatsfortestingonly", extra="extra"
            )
        assert "Extra inputs are not permitted [type=extra_forbidden, input_value='extra', input_type=str]" in str(
            e.value
        )

    def test_serialization(self, openai_llm_config_entry: OpenAILLMConfigEntry) -> None:
        actual = openai_llm_config_entry.model_dump()
        expected = {
            "api_type": "openai",
            "model": "gpt-4o-mini",
            "api_key": "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
            "tags": [],
        }
        assert actual == expected

    def test_deserialization(self, openai_llm_config_entry: OpenAILLMConfigEntry) -> None:
        actual = OpenAILLMConfigEntry(**openai_llm_config_entry.model_dump())
        assert actual == openai_llm_config_entry

    def test_get(self, openai_llm_config_entry: OpenAILLMConfigEntry) -> None:
        assert openai_llm_config_entry.get("api_type") == "openai"
        assert openai_llm_config_entry.get("model") == "gpt-4o-mini"
        assert openai_llm_config_entry.get("api_key") == "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly"
        assert openai_llm_config_entry.get("doesnt_exists") is None
        assert openai_llm_config_entry.get("doesnt_exists", "default") == "default"

    def test_get_item_and_set_item(self, openai_llm_config_entry: OpenAILLMConfigEntry) -> None:
        # Test __getitem__
        assert openai_llm_config_entry["api_type"] == "openai"
        assert openai_llm_config_entry["model"] == "gpt-4o-mini"
        assert openai_llm_config_entry["api_key"] == "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly"
        assert openai_llm_config_entry["tags"] == []
        with pytest.raises(KeyError) as e:
            openai_llm_config_entry["wrong_key"]
        assert str(e.value) == "\"Key 'wrong_key' not found in OpenAILLMConfigEntry\""

        # Test __setitem__
        assert openai_llm_config_entry["base_url"] is None
        openai_llm_config_entry["base_url"] = "https://api.openai.com"
        assert openai_llm_config_entry["base_url"] == "https://api.openai.com"
        openai_llm_config_entry["base_url"] = None
        assert openai_llm_config_entry["base_url"] is None


class TestLLMConfig:
    @pytest.fixture
    def openai_llm_config(self, openai_llm_config_entry: OpenAILLMConfigEntry) -> LLMConfig:
        return LLMConfig(config_list=[openai_llm_config_entry], temperature=0.5, check_every_ms=1000, cache_seed=42)

    @pytest.mark.parametrize(
        "llm_config, expected",
        [
            (
                # todo add more test cases
                {
                    "config_list": [
                        {"model": "gpt-4o-mini", "api_key": "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly"}
                    ]
                },
                LLMConfig(
                    config_list=[
                        OpenAILLMConfigEntry(
                            model="gpt-4o-mini",
                            api_key="sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                        )
                    ]
                ),
            ),
            (
                {"model": "gpt-4o-mini", "api_key": "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly"},
                LLMConfig(
                    config_list=[
                        OpenAILLMConfigEntry(
                            model="gpt-4o-mini",
                            api_key="sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                        )
                    ]
                ),
            ),
            (
                {
                    "model": "gpt-4o-mini",
                    "api_key": "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                    "cache_seed": 42,
                },
                LLMConfig(
                    config_list=[
                        OpenAILLMConfigEntry(
                            model="gpt-4o-mini",
                            api_key="sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                        )
                    ],
                    cache_seed=42,
                ),
            ),
            (
                {
                    "config_list": [
                        {"model": "gpt-4o-mini", "api_key": "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly"}
                    ],
                    "max_tokens": 1024,
                },
                LLMConfig(
                    config_list=[
                        OpenAILLMConfigEntry(
                            model="gpt-4o-mini",
                            api_key="sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                            max_tokens=1024,
                        )
                    ]
                ),
            ),
            (
                {
                    "config_list": [
                        {
                            "model": "gpt-4o-mini",
                            "api_key": "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                            "api_type": "openai",
                        }
                    ],
                    "temperature": 0.5,
                    "check_every_ms": 1000,
                    "cache_seed": 42,
                },
                LLMConfig(
                    config_list=[
                        OpenAILLMConfigEntry(
                            model="gpt-4o-mini",
                            api_key="sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                        )
                    ],
                    temperature=0.5,
                    check_every_ms=1000,
                    cache_seed=42,
                ),
            ),
            (
                {
                    "config_list": [
                        {
                            "model": "gpt-4o-mini",
                            "api_key": "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                            "api_type": "azure",
                            "base_url": "https://api.openai.com",
                        }
                    ],
                },
                LLMConfig(
                    config_list=[
                        AzureOpenAILLMConfigEntry(
                            model="gpt-4o-mini",
                            api_key="sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                            base_url="https://api.openai.com",
                        )
                    ],
                ),
            ),
            (
                {
                    "config_list": [
                        {
                            "api_type": "anthropic",
                            "model": "claude-3-5-sonnet-latest",
                            "api_key": "dummy_api_key",
                            "stream": False,
                            "temperature": 1.0,
                            "top_p": 0.8,
                            "max_tokens": 100,
                            "tags": [],
                        }
                    ]
                },
                LLMConfig(
                    config_list=[
                        AnthropicLLMConfigEntry(
                            model="claude-3-5-sonnet-latest",
                            api_key="dummy_api_key",
                            stream=False,
                            temperature=1.0,
                            top_p=0.8,
                            max_tokens=100,
                        )
                    ],
                ),
            ),
            (
                {
                    "config_list": [
                        {
                            "api_type": "bedrock",
                            "model": "anthropic.claude-3-sonnet-20240229-v1:0",
                            "aws_region": "us-east-1",
                            "aws_access_key": "test_access_key_id",
                            "aws_secret_key": "test_secret_access_key",
                            "aws_session_token": "test_session_token",
                            "temperature": 0.8,
                            "topP": 0.6,
                            "stream": False,
                            "tags": [],
                            "supports_system_prompts": True,
                        }
                    ]
                },
                LLMConfig(
                    config_list=[
                        BedrockLLMConfigEntry(
                            model="anthropic.claude-3-sonnet-20240229-v1:0",
                            aws_region="us-east-1",
                            aws_access_key="test_access_key_id",
                            aws_secret_key="test_secret_access_key",
                            aws_session_token="test_session_token",
                            temperature=0.8,
                            topP=0.6,
                            stream=False,
                        )
                    ]
                ),
            ),
            (
                {
                    "config_list": [
                        {
                            "api_type": "cerebras",
                            "api_key": "fake_api_key",
                            "model": "llama3.1-8b",
                            "max_tokens": 1000,
                            "seed": 42,
                            "stream": False,
                            "temperature": 1.0,
                            "tags": [],
                        }
                    ]
                },
                LLMConfig(
                    config_list=[
                        CerebrasLLMConfigEntry(
                            api_key="fake_api_key",
                            model="llama3.1-8b",
                            max_tokens=1000,
                            seed=42,
                            stream=False,
                            temperature=1,
                        )
                    ]
                ),
            ),
            (
                {
                    "config_list": [
                        {
                            "api_type": "cohere",
                            "model": "command-r-plus",
                            "api_key": "dummy_api_key",
                            "frequency_penalty": 0,
                            "k": 0,
                            "p": 0.75,
                            "presence_penalty": 0,
                            "strict_tools": False,
                            "tags": [],
                            "temperature": 0.3,
                        }
                    ]
                },
                LLMConfig(
                    config_list=[
                        CohereLLMConfigEntry(
                            model="command-r-plus",
                            api_key="dummy_api_key",
                            stream=False,
                        )
                    ]
                ),
            ),
            (
                {
                    "config_list": [
                        {
                            "api_type": "deepseek",
                            "api_key": "fake_api_key",
                            "model": "deepseek-chat",
                            "base_url": "https://api.deepseek.com/v1",
                            "max_tokens": 8192,
                            "temperature": 0.5,
                            "tags": [],
                        }
                    ]
                },
                LLMConfig(
                    config_list=[
                        DeepSeekLLMConfigEntry(
                            api_key="fake_api_key",
                            model="deepseek-chat",
                        )
                    ]
                ),
            ),
            (
                {
                    "config_list": [
                        {
                            "api_type": "google",
                            "model": "gemini-2.0-flash-lite",
                            "api_key": "dummy_api_key",
                            "project_id": "fake-project-id",
                            "location": "us-west1",
                            "stream": False,
                            "tags": [],
                        }
                    ]
                },
                LLMConfig(
                    config_list=[
                        GeminiLLMConfigEntry(
                            model="gemini-2.0-flash-lite",
                            api_key="dummy_api_key",
                            project_id="fake-project-id",
                            location="us-west1",
                        )
                    ]
                ),
            ),
            (
                {
                    "config_list": [
                        {
                            "api_type": "groq",
                            "model": "llama3-8b-8192",
                            "api_key": "fake_api_key",
                            "temperature": 1,
                            "stream": False,
                            "tags": [],
                        }
                    ]
                },
                LLMConfig(config_list=[GroqLLMConfigEntry(api_key="fake_api_key", model="llama3-8b-8192")]),
            ),
            (
                {
                    "config_list": [
                        {
                            "api_type": "mistral",
                            "model": "mistral-small-latest",
                            "api_key": "fake_api_key",
                            "safe_prompt": False,
                            "stream": False,
                            "temperature": 0.7,
                            "tags": [],
                        }
                    ]
                },
                LLMConfig(
                    config_list=[
                        MistralLLMConfigEntry(
                            model="mistral-small-latest",
                            api_key="fake_api_key",
                        )
                    ]
                ),
            ),
            (
                {
                    "config_list": [
                        {
                            "api_type": "ollama",
                            "model": "llama3.1:8b",
                            "num_ctx": 2048,
                            "num_predict": -1,
                            "repeat_penalty": 1.1,
                            "seed": 0,
                            "stream": False,
                            "tags": [],
                            "temperature": 0.8,
                            "top_k": 40,
                            "top_p": 0.9,
                        }
                    ]
                },
                LLMConfig(config_list=[OllamaLLMConfigEntry(model="llama3.1:8b")]),
            ),
            (
                {
                    "config_list": [
                        {
                            "api_type": "together",
                            "model": "mistralai/Mixtral-8x7B-Instruct-v0.1",
                            "api_key": "fake_api_key",
                            "safety_model": "Meta-Llama/Llama-Guard-7b",
                            "tags": [],
                            "max_tokens": 512,
                            "stream": False,
                        }
                    ]
                },
                LLMConfig(
                    config_list=[
                        TogetherLLMConfigEntry(
                            model="mistralai/Mixtral-8x7B-Instruct-v0.1",
                            api_key="fake_api_key",
                            safety_model="Meta-Llama/Llama-Guard-7b",
                        )
                    ]
                ),
            ),
            (
                {
                    "model": "gpt-4o-realtime-preview",
                    "api_key": "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                    "voice": "alloy",
                    "tags": ["gpt-4o-realtime", "realtime"],
                },
                LLMConfig(
                    model="gpt-4o-realtime-preview",
                    api_key="sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                    voice="alloy",
                    tags=["gpt-4o-realtime", "realtime"],
                ),
            ),
        ],
    )
    def test_init(self, llm_config: dict[str, Any], expected: LLMConfig) -> None:
        actual = LLMConfig(**llm_config)
        assert actual == expected, actual

    def test_extra_fields(self) -> None:
        with pytest.raises(ValueError) as e:
            LLMConfig(
                config_list=[
                    OpenAILLMConfigEntry(
                        model="gpt-4o-mini", api_key="sk-mockopenaiAPIkeysinexpectedformatsfortestingonly"
                    )
                ],
                extra="extra",
            )
        assert "Extra inputs are not permitted [type=extra_forbidden, input_value='extra', input_type=str]" in str(
            e.value
        )

    def test_serialization(self, openai_llm_config: LLMConfig) -> None:
        actual = openai_llm_config.model_dump()
        expected = {
            "config_list": [
                {
                    "api_type": "openai",
                    "model": "gpt-4o-mini",
                    "api_key": "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                    "tags": [],
                }
            ],
            "temperature": 0.5,
            "check_every_ms": 1000,
            "cache_seed": 42,
        }
        assert actual == expected

    def test_deserialization(self, openai_llm_config: LLMConfig) -> None:
        actual = LLMConfig(**openai_llm_config.model_dump())
        assert actual.model_dump() == openai_llm_config.model_dump()
        assert type(actual._model) == type(openai_llm_config._model)
        assert actual._model == openai_llm_config._model
        assert actual == openai_llm_config
        assert isinstance(actual.config_list[0], OpenAILLMConfigEntry)

    def test_get(self, openai_llm_config: LLMConfig) -> None:
        assert openai_llm_config.get("temperature") == 0.5
        assert openai_llm_config.get("check_every_ms") == 1000
        assert openai_llm_config.get("cache_seed") == 42
        assert openai_llm_config.get("doesnt_exists") is None
        assert openai_llm_config.get("doesnt_exists", "default") == "default"

    def test_getattr(self, openai_llm_config: LLMConfig) -> None:
        assert openai_llm_config.temperature == 0.5
        assert openai_llm_config.check_every_ms == 1000
        assert openai_llm_config.cache_seed == 42
        assert openai_llm_config.config_list == [openai_llm_config.config_list[0]]
        with pytest.raises(AttributeError) as e:
            openai_llm_config.wrong_key
        assert str(e.value) == "'LLMConfig' object has no attribute 'wrong_key'"

    def test_setattr(self, openai_llm_config: LLMConfig) -> None:
        assert openai_llm_config.temperature == 0.5
        openai_llm_config.temperature = 0.8
        assert openai_llm_config.temperature == 0.8

    def test_get_item_and_set_item(self, openai_llm_config: LLMConfig) -> None:
        # Test __getitem__
        assert openai_llm_config["temperature"] == 0.5
        assert openai_llm_config["check_every_ms"] == 1000
        assert openai_llm_config["cache_seed"] == 42
        assert openai_llm_config["config_list"] == [openai_llm_config.config_list[0]]
        with pytest.raises(KeyError) as e:
            openai_llm_config["wrong_key"]
        assert str(e.value) == "\"Key 'wrong_key' not found in LLMConfig\""

        # Test __setitem__
        assert openai_llm_config["timeout"] is None
        openai_llm_config["timeout"] = 60
        assert openai_llm_config["timeout"] == 60
        openai_llm_config["timeout"] = None
        assert openai_llm_config["timeout"] is None

    def test_items(self, openai_llm_config: LLMConfig) -> None:
        actual = openai_llm_config.items()  # type: ignore[var-annotated]
        assert isinstance(actual, dict_items)
        expected = {
            "config_list": [
                {
                    "api_type": "openai",
                    "model": "gpt-4o-mini",
                    "api_key": "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                    "tags": [],
                }
            ],
            "temperature": 0.5,
            "check_every_ms": 1000,
            "cache_seed": 42,
        }
        assert dict(actual) == expected, dict(actual)

    def test_keys(self, openai_llm_config: LLMConfig) -> None:
        actual = openai_llm_config.keys()  # type: ignore[var-annotated]
        assert isinstance(actual, dict_keys)
        expected = ["temperature", "check_every_ms", "cache_seed", "config_list"]
        assert list(actual) == expected, list(actual)

    def test_values(self, openai_llm_config: LLMConfig) -> None:
        actual = openai_llm_config.values()  # type: ignore[var-annotated]
        assert isinstance(actual, dict_values)
        expected = [
            0.5,
            1000,
            42,
            [
                {
                    "api_type": "openai",
                    "model": "gpt-4o-mini",
                    "api_key": "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                    "tags": [],
                }
            ],
        ]
        assert list(actual) == expected, list(actual)

    def test_unpack(self, openai_llm_config: LLMConfig, openai_llm_config_entry: OpenAILLMConfigEntry) -> None:
        openai_llm_config_entry.base_url = "localhost:8080"  # type: ignore[assignment]
        openai_llm_config.config_list = [  # type: ignore[attr-defined]
            openai_llm_config_entry,
        ]
        expected = {
            "config_list": [
                {
                    "api_type": "openai",
                    "model": "gpt-4o-mini",
                    "api_key": "sk-mockopenaiAPIkeysinexpectedformatsfortestingonly",
                    "base_url": "localhost:8080",
                    "tags": [],
                }
            ],
            "temperature": 0.5,
            "check_every_ms": 1000,
            "cache_seed": 42,
        }

        def test_unpacking(**kwargs: Any) -> None:
            for k, v in expected.items():
                assert k in kwargs
                if k == "config_list":
                    assert kwargs[k][0].model_dump() == v[0]  # type: ignore[index]
                else:
                    assert kwargs[k] == v
            # assert kwargs == expected, kwargs

        test_unpacking(**openai_llm_config)

    def test_contains(self, openai_llm_config: LLMConfig) -> None:
        assert "temperature" in openai_llm_config
        assert "check_every_ms" in openai_llm_config
        assert "cache_seed" in openai_llm_config
        assert "config_list" in openai_llm_config
        assert "doesnt_exists" not in openai_llm_config
        assert "config_list" in openai_llm_config
        assert not "config_list" not in openai_llm_config

    def test_with_context(self, openai_llm_config: LLMConfig) -> None:
        # Test with dummy agent
        class DummyAgent:
            def __init__(self) -> None:
                self.llm_config = LLMConfig.get_current_llm_config()

        with openai_llm_config:
            agent = DummyAgent()
        assert agent.llm_config == openai_llm_config
        assert agent.llm_config.temperature == 0.5
        assert agent.llm_config.config_list[0]["model"] == "gpt-4o-mini"

        # Test passing LLMConfig object as parameter
        assert LLMConfig.get_current_llm_config(openai_llm_config) == openai_llm_config

        # Test accessing current_llm_config outside the context
        assert LLMConfig.get_current_llm_config() is None
        with openai_llm_config:
            actual = LLMConfig.get_current_llm_config()
            assert actual == openai_llm_config

        assert LLMConfig.get_current_llm_config() is None

    @pytest.mark.parametrize(
        "filter_dict, exclude, expected",
        [
            (
                {"tags": ["gpt35", "gpt4"]},
                False,
                JSON_SAMPLE_DICT[0:2],
            ),
            (
                {"tags": ["gpt35", "gpt4"]},
                True,
                JSON_SAMPLE_DICT[2:4],
            ),
            (
                {"api_type": "azure", "api_version": "2024-02-01"},
                False,
                [JSON_SAMPLE_DICT[2]],
            ),
            (
                {"api_type": ["azure"]},
                False,
                [JSON_SAMPLE_DICT[2]],
            ),
            (
                {},
                False,
                JSON_SAMPLE_DICT,
            ),
        ],
    )
    def test_where(self, filter_dict: dict[str, Any], exclude: bool, expected: list[dict[str, Any]]) -> None:
        openai_llm_config = LLMConfig(config_list=JSON_SAMPLE_DICT)

        actual = openai_llm_config.where(**filter_dict, exclude=exclude)
        assert isinstance(actual, LLMConfig)
        assert actual.config_list == LLMConfig(config_list=expected).config_list

    def test_where_invalid_filter(self) -> None:
        openai_llm_config = LLMConfig(config_list=JSON_SAMPLE_DICT)

        with pytest.raises(ValueError) as e:
            openai_llm_config.where(api_type="invalid")
        assert str(e.value) == "No config found that satisfies the filter criteria: {'api_type': 'invalid'}"

    def test_repr(self, openai_llm_config: LLMConfig) -> None:
        actual = repr(openai_llm_config)
        expected = "LLMConfig(temperature=0.5, check_every_ms=1000, cache_seed=42, config_list=[{'api_type': 'openai', 'model': 'gpt-4o-mini', 'api_key': '**********', 'tags': []}])"
        assert actual == expected, actual

    def test_str(self, openai_llm_config: LLMConfig) -> None:
        actual = str(openai_llm_config)
        expected = "LLMConfig(temperature=0.5, check_every_ms=1000, cache_seed=42, config_list=[{'api_type': 'openai', 'model': 'gpt-4o-mini', 'api_key': '**********', 'tags': []}])"
        assert actual == expected, actual

    def test_from_json_env(self, monkeypatch: pytest.MonkeyPatch) -> None:
        monkeypatch.setenv("LLM_CONFIG", JSON_SAMPLE)
        expected = LLMConfig(config_list=JSON_SAMPLE_DICT)
        actual = LLMConfig.from_json(env="LLM_CONFIG")
        assert isinstance(actual, LLMConfig)
        assert actual == expected, actual

    @pytest.mark.xfail(reason="Currently raises FileNotFoundError")
    def test_from_json_env_not_found(self) -> None:
        with pytest.raises(ValueError) as e:
            LLMConfig.from_json(env="INVALID_ENV")
        assert str(e.value) == "Environment variable 'INVALID_ENV' not found"

    def test_from_json_env_with_kwargs(self, monkeypatch: pytest.MonkeyPatch) -> None:
        monkeypatch.setenv("LLM_CONFIG", JSON_SAMPLE)
        expected = LLMConfig(config_list=JSON_SAMPLE_DICT, temperature=0.5, check_every_ms=1000, cache_seed=42)
        actual = LLMConfig.from_json(env="LLM_CONFIG", temperature=0.5, check_every_ms=1000, cache_seed=42)
        assert isinstance(actual, LLMConfig)
        assert actual == expected, actual

    def test_from_json_path(self) -> None:
        with tempfile.TemporaryDirectory() as tmpdirname:
            file_path = f"{tmpdirname}/llm_config.json"
            with open(file_path, "w") as f:
                f.write(JSON_SAMPLE)

            expected = LLMConfig(config_list=JSON_SAMPLE_DICT)
            actual = LLMConfig.from_json(path=file_path)
            assert isinstance(actual, LLMConfig)
            assert actual == expected, actual

        with pytest.raises(FileNotFoundError) as e:
            LLMConfig.from_json(path="invalid_path")
        assert "No such file or directory: 'invalid_path'" in str(e.value)

    def test_copy(self, openai_llm_config: LLMConfig) -> None:
        actual = openai_llm_config.copy()
        assert actual == openai_llm_config
        assert actual is not openai_llm_config

        actual = openai_llm_config.deepcopy()
        assert actual == openai_llm_config
        assert actual is not openai_llm_config

        actual = copy(openai_llm_config)
        assert actual == openai_llm_config
        assert actual is not openai_llm_config

        actual = deepcopy(openai_llm_config)
        assert actual == openai_llm_config
        assert actual is not openai_llm_config

    def test_current(self) -> None:
        llm_config = LLMConfig(config_list=JSON_SAMPLE_DICT)

        # Test without context. Should raise an error
        expected_error = "No current LLMConfig set. Are you inside a context block?"
        with pytest.raises(ValueError) as e:
            LLMConfig.current
        assert str(e.value) == expected_error
        with pytest.raises(ValueError) as e:
            LLMConfig.default
        assert str(e.value) == expected_error

        with llm_config:
            assert LLMConfig.get_current_llm_config() == llm_config
            assert LLMConfig.current == llm_config
            assert LLMConfig.default == llm_config

            with LLMConfig.current.where(api_type="openai"):
                assert LLMConfig.get_current_llm_config() == llm_config.where(api_type="openai")
                assert LLMConfig.current == llm_config.where(api_type="openai")
                assert LLMConfig.default == llm_config.where(api_type="openai")

                with LLMConfig.default.where(model="gpt-4"):
                    assert LLMConfig.get_current_llm_config() == llm_config.where(api_type="openai", model="gpt-4")
                    assert LLMConfig.current == llm_config.where(api_type="openai", model="gpt-4")
                    assert LLMConfig.default == llm_config.where(api_type="openai", model="gpt-4")



# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
#
# SPDX-License-Identifier: Apache-2.0

import re
import sys
from types import ModuleType
from typing import Any, Iterable, Iterator, Optional, Type, Union

import pytest

from autogen.import_utils import ModuleInfo, get_missing_imports, optional_import_block, require_optional_import


@pytest.fixture
def mock_module() -> Iterator[ModuleType]:
    module_name = "mock_module"
    module = ModuleType(module_name)
    module.__version__ = "1.0.0"  # type: ignore[attr-defined]
    sys.modules[module_name] = module
    yield module
    del sys.modules[module_name]


@pytest.fixture
def mock_module_without_version() -> Iterator[ModuleType]:
    module_name = "mock_module"
    module = ModuleType(module_name)
    sys.modules[module_name] = module
    yield module
    del sys.modules[module_name]


class MockModule:
    def __init__(self, name: str, version: str):
        self.__name__ = name
        self.__version__ = version


@pytest.fixture
def mock_modules() -> Iterator[dict[str, ModuleType]]:
    modules = {
        "module_a": MockModule("module_a", "1.0.0"),
        "module_b": MockModule("module_b", "2.0.0"),
        "module_c": MockModule("module_c", "3.0.0"),
    }
    original_sys_modules = sys.modules.copy()
    sys.modules.update(modules)  # type: ignore[arg-type]
    assert all(module in sys.modules for module in modules)
    try:
        yield modules  # type: ignore[misc]
    finally:
        sys.modules.clear()
        sys.modules.update(original_sys_modules)


class TestmoduleInfo:
    @pytest.mark.parametrize(
        "module_info, expected",
        [
            (
                "jupyter-client>=8.6.0,<9.0.0",
                ModuleInfo(
                    name="jupyter-client",
                    min_version="8.6.0",
                    max_version="9.0.0",
                    min_inclusive=True,
                    max_inclusive=False,
                ),
            ),
            (
                "jupyter-client>=8.6.0",
                ModuleInfo(
                    name="jupyter-client",
                    min_version="8.6.0",
                    max_version=None,
                    min_inclusive=True,
                    max_inclusive=False,
                ),
            ),
            (
                "jupyter-client<9.0.0",
                ModuleInfo(
                    name="jupyter-client",
                    min_version=None,
                    max_version="9.0.0",
                    min_inclusive=False,
                    max_inclusive=False,
                ),
            ),
            (
                "jupyter-client",
                ModuleInfo(
                    name="jupyter-client", min_version=None, max_version=None, min_inclusive=False, max_inclusive=False
                ),
            ),
        ],
    )
    def test_from_str_success(self, module_info: str, expected: ModuleInfo) -> None:
        result = ModuleInfo.from_str(module_info)
        assert result == expected

    def test_from_str_with_invalid_format(self) -> None:
        module_info = "jupyter-client>="
        with pytest.raises(ValueError, match="Invalid module information: jupyter-client>="):
            ModuleInfo.from_str(module_info)

    @pytest.mark.parametrize(
        "module_info, expected",
        [
            (ModuleInfo(name="mock_module"), None),
            (ModuleInfo(name="non_existent_module"), "'non_existent_module' is not installed."),
            (ModuleInfo(name="mock_module", min_version="1.0.0", min_inclusive=True), None),
            (
                ModuleInfo(name="mock_module", min_version="1.0.0", min_inclusive=False),
                "'mock_module' is installed, but the installed version 1.0.0 is too low (required 'mock_module>1.0.0').",
            ),
            (ModuleInfo(name="mock_module", min_version="0.9.0", min_inclusive=True), None),
            (
                ModuleInfo(name="mock_module", min_version="1.1.0", min_inclusive=True),
                "'mock_module' is installed, but the installed version 1.0.0 is too low (required 'mock_module>=1.1.0').",
            ),
            (ModuleInfo(name="mock_module", max_version="1.0.0", max_inclusive=True), None),
            (
                ModuleInfo(name="mock_module", max_version="1.0.0", max_inclusive=False),
                "'mock_module' is installed, but the installed version 1.0.0 is too high (required 'mock_module<1.0.0').",
            ),
            (ModuleInfo(name="mock_module", max_version="1.1.0", max_inclusive=True), None),
            (
                ModuleInfo(name="mock_module", max_version="0.9.0", max_inclusive=True),
                "'mock_module' is installed, but the installed version 1.0.0 is too high (required 'mock_module<=0.9.0').",
            ),
            (
                ModuleInfo(
                    name="mock_module", min_version="0.9.0", max_version="1.1.0", min_inclusive=True, max_inclusive=True
                ),
                None,
            ),
            (
                ModuleInfo(
                    name="mock_module", min_version="1.0.0", max_version="1.0.0", min_inclusive=True, max_inclusive=True
                ),
                None,
            ),
            (
                ModuleInfo(
                    name="mock_module",
                    min_version="1.0.0",
                    max_version="1.0.0",
                    min_inclusive=False,
                    max_inclusive=False,
                ),
                "'mock_module' is installed, but the installed version 1.0.0 is too low (required 'mock_module>1.0.0<1.0.0').",
            ),
        ],
    )
    def test_is_in_sys_modules(self, mock_module: ModuleType, module_info: ModuleInfo, expected: Optional[str]) -> None:
        assert module_info.is_in_sys_modules() == expected

    @pytest.mark.parametrize(
        "module_info, expected",
        [
            (ModuleInfo(name="mock_module"), None),
            (
                ModuleInfo(name="mock_module", min_version="1.0.0", min_inclusive=True),
                "'mock_module' is installed, but the version is not available.",
            ),
        ],
    )
    def test_is_in_sys_modules_without_version(
        self, mock_module_without_version: ModuleType, module_info: ModuleInfo, expected: Optional[str]
    ) -> None:
        assert module_info.is_in_sys_modules() == expected


class TestOptionalImportBlock:
    def test_optional_import_block(self) -> None:
        with optional_import_block():
            import ast

            import some_module
            import some_other_module

        assert ast is not None
        with pytest.raises(
            UnboundLocalError,
            match=r"(local variable 'some_module' referenced before assignment|cannot access local variable 'some_module' where it is not associated with a value)",
        ):
            some_module

        with pytest.raises(
            UnboundLocalError,
            match=r"(local variable 'some_other_module' referenced before assignment|cannot access local variable 'some_other_module' where it is not associated with a value)",
        ):
            some_other_module


class TestRequiresOptionalImportCallables:
    def test_version_too_high(self, mock_module: ModuleInfo) -> None:
        assert mock_module.__name__ in sys.modules  # type: ignore[attr-defined]
        mock_info = f"{mock_module.__name__}>{mock_module.__version__}"  # type: ignore[attr-defined]

        @require_optional_import(mock_info, "mock-module")
        def dummy_function() -> None:
            """Dummy function to test requires_optional_import"""
            pass

        with pytest.raises(
            ImportError,
            match=re.escape("""A module needed for test.test_import_utils.dummy_function is missing:
 - 'mock_module' is installed, but the installed version 1.0.0 is too low (required 'mock_module>1.0.0').
Please install it using:
'pip install ag2[mock-module]'"""),
        ):
            dummy_function()

    @pytest.mark.parametrize("except_for", [None, "dummy_function", ["dummy_function"]])
    def test_function_attributes(self, except_for: Optional[Union[str, list[str]]]) -> None:
        def dummy_function() -> None:
            """Dummy function to test requires_optional_import"""
            pass

        dummy_function.__module__ = "some_random_module.dummy_stuff"

        actual = require_optional_import("some_optional_module", "optional_dep", except_for=except_for)(dummy_function)

        assert actual is not None
        assert actual.__module__ == "some_random_module.dummy_stuff"
        assert actual.__name__ == "dummy_function"
        assert actual.__doc__ == "Dummy function to test requires_optional_import"

        if not except_for:
            with pytest.raises(
                ImportError,
                match=re.escape("""A module needed for some_random_module.dummy_stuff.dummy_function is missing:
 - 'some_optional_module' is not installed.
Please install it using:
'pip install ag2[optional_dep]"""),
            ):
                actual()
        else:
            actual()

    @pytest.mark.parametrize("except_for", [None, "dummy_function", ["dummy_function"]])
    def test_function_call(self, except_for: Optional[Union[str, list[str]]]) -> None:
        @require_optional_import("some_optional_module", "optional_dep", except_for=except_for)
        def dummy_function() -> None:
            """Dummy function to test requires_optional_import"""
            pass

        if not except_for:
            with pytest.raises(
                ImportError,
                match=re.escape("""A module needed for test.test_import_utils.dummy_function is missing:
 - 'some_optional_module' is not installed.
Please install it using:
'pip install ag2[optional_dep]'"""),
            ):
                dummy_function()
        else:
            dummy_function()

    @pytest.mark.parametrize("except_for", [None, "dummy_method", ["dummy_method"]])
    def test_method_attributes(self, except_for: Optional[Union[str, list[str]]]) -> None:
        class DummyClass:
            def dummy_method(self) -> None:
                """Dummy method to test requires_optional_import"""
                pass

        assert hasattr(DummyClass.dummy_method, "__module__")
        assert DummyClass.dummy_method.__module__ == "test.test_import_utils"

        DummyClass.__module__ = "some_random_module.dummy_stuff"
        DummyClass.dummy_method.__module__ = "some_random_module.dummy_stuff"

        DummyClass.dummy_method = require_optional_import(  # type: ignore[method-assign]
            "some_optional_module", "optional_dep", except_for=except_for
        )(DummyClass.dummy_method)

        assert DummyClass.dummy_method is not None
        assert DummyClass.dummy_method.__module__ == "some_random_module.dummy_stuff"
        assert DummyClass.dummy_method.__name__ == "dummy_method"
        assert DummyClass.dummy_method.__doc__ == "Dummy method to test requires_optional_import"

        dummy = DummyClass()

        if not except_for:
            with pytest.raises(
                ImportError,
                match=re.escape("""A module needed for some_random_module.dummy_stuff.dummy_method is missing:
 - 'some_optional_module' is not installed.
Please install it using:
'pip install ag2[optional_dep]'"""),
            ):
                dummy.dummy_method()
        else:
            dummy.dummy_method()

    @pytest.mark.parametrize("except_for", [None, "dummy_method", ["dummy_method"]])
    def test_method_call(self, except_for: Optional[Union[str, list[str]]]) -> None:
        class DummyClass:
            @require_optional_import("some_optional_module", "optional_dep", except_for=except_for)
            def dummy_method(self) -> None:
                """Dummy method to test requires_optional_import"""
                pass

        dummy = DummyClass()

        if not except_for:
            with pytest.raises(
                ImportError,
                match=re.escape("""A module needed for test.test_import_utils.dummy_method is missing:
 - 'some_optional_module' is not installed.
Please install it using:
'pip install ag2[optional_dep]'"""),
            ):
                dummy.dummy_method()
        else:
            dummy.dummy_method()

    @pytest.mark.parametrize("except_for", [None, "dummy_static_function", ["dummy_static_function"]])
    def test_static_call(self, except_for: Optional[Union[str, list[str]]]) -> None:
        class DummyClass:
            @require_optional_import("some_optional_module", "optional_dep", except_for=except_for)
            @staticmethod
            def dummy_static_function() -> None:
                """Dummy static function to test requires_optional_import"""
                pass

        dummy = DummyClass()

        if not except_for:
            with pytest.raises(
                ImportError,
                match=re.escape("""A module needed for test.test_import_utils.dummy_static_function is missing:
 - 'some_optional_module' is not installed.
Please install it using:
'pip install ag2[optional_dep]'"""),
            ):
                dummy.dummy_static_function()
        else:
            dummy.dummy_static_function()

    @pytest.mark.parametrize("except_for", [None, "dummy_property", ["dummy_property"]])
    def test_property_call(self, except_for: Optional[Union[str, list[str]]]) -> None:
        class DummyClass:
            @property
            @require_optional_import("some_optional_module", "optional_dep", except_for=except_for)
            def dummy_property(self) -> int:
                """Dummy property to test requires_optional_import"""
                return 4

        dummy = DummyClass()

        if not except_for:
            with pytest.raises(
                ImportError,
                match=re.escape("""A module needed for test.test_import_utils.dummy_property is missing:
 - 'some_optional_module' is not installed.
Please install it using:
'pip install ag2[optional_dep]'"""),
            ):
                dummy.dummy_property

        else:
            dummy.dummy_property


class TestRequiresOptionalImportClasses:
    @pytest.fixture
    def dummy_cls(self) -> Type[Any]:
        @require_optional_import("some_optional_module", "optional_dep")
        class DummyClass:
            def dummy_method(self) -> None:
                """Dummy method to test requires_optional_import"""
                pass

            @staticmethod
            def dummy_static_method() -> None:
                """Dummy static method to test requires_optional_import"""
                pass

            @classmethod
            def dummy_class_method(cls) -> None:
                """Dummy class method to test requires_optional_import"""
                pass

            @property
            def dummy_property(self) -> int:
                """Dummy property to test requires_optional_import"""
                return 4

        return DummyClass

    def test_class_init_call(self, dummy_cls: Type[Any]) -> None:
        with pytest.raises(
            ImportError,
            match=re.escape("""A module needed for __init__ is missing:
 - 'some_optional_module' is not installed.
Please install it using:\n'pip install ag2[optional_dep]'"""),
        ):
            dummy_cls()


class TestGetMissingImports:
    @pytest.mark.parametrize(
        "modules, expected_missing",
        [
            (["module_a", "module_b", "module_c"], {}),
            (["module_a>=1.0.0", "module_b>=2.0.0", "module_c>=3.0.0"], {}),
            (
                ["module_a>=1.0.1", "module_b>=2.0.0", "module_c>=3.0.0"],
                {
                    "module_a": "'module_a' is installed, but the installed version 1.0.0 is too low (required 'module_a>=1.0.1')."
                },
            ),
            (
                ["module_a>=1.0.0", "module_b>=2.1.0", "module_c>=3.0.0"],
                {
                    "module_b": "'module_b' is installed, but the installed version 2.0.0 is too low (required 'module_b>=2.1.0')."
                },
            ),
            (
                ["module_a>=1.0.0", "module_b>=2.0.0", "module_c>=3.1.0"],
                {
                    "module_c": "'module_c' is installed, but the installed version 3.0.0 is too low (required 'module_c>=3.1.0')."
                },
            ),
            (["module_a>=1.0.0", "module_b>=2.0.0", "module_d"], {"module_d": "'module_d' is not installed."}),
            (
                ["module_a>=1.0.0", "module_b>=2.0.0", "module_c<3.0.0"],
                {
                    "module_c": "'module_c' is installed, but the installed version 3.0.0 is too high (required 'module_c<3.0.0')."
                },
            ),
            (["module_a>=1.0.0", "module_b>=2.0.0", "module_c<=3.0.0"], {}),
            (
                ["module_a>=1.0.0", "module_b>=2.0.0", "module_c>3.0.0"],
                {
                    "module_c": "'module_c' is installed, but the installed version 3.0.0 is too low (required 'module_c>3.0.0')."
                },
            ),
            (["module_a>=1.0.0", "module_b>=2.0.0", "module_c<3.1.0"], {}),
        ],
    )
    def test_get_missing_imports(
        self, mock_modules: dict[str, MockModule], modules: Union[str, Iterable[str]], expected_missing: dict[str, str]
    ) -> None:
        assert mock_modules
        missing = get_missing_imports(modules)
        assert missing == expected_missing



# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
#
# SPDX-License-Identifier: Apache-2.0
#
# Portions derived from  https://github.com/microsoft/autogen are under the MIT License.
# SPDX-License-Identifier: MIT


import os
import subprocess

import pytest

from .conftest import Credentials, Secrets, credentials_all_llms, suppress_gemini_resource_exhausted


@pytest.mark.parametrize("credentials_from_test_param", credentials_all_llms, indirect=True)
@suppress_gemini_resource_exhausted
def test_credentials_from_test_param_fixture(
    credentials_from_test_param: Credentials,
    request: pytest.FixtureRequest,
) -> None:
    # Get the parameter name request node
    current_llm = request.node.callspec.id

    assert current_llm is not None
    assert isinstance(credentials_from_test_param, Credentials)

    first_config = credentials_from_test_param.config_list[0]
    if "gpt_4" in current_llm:
        if "api_type" in first_config:
            assert first_config["api_type"] == "openai"
    elif "gemini" in current_llm:
        assert first_config["api_type"] == "google"
    elif "anthropic" in current_llm:
        assert first_config["api_type"] == "anthropic"
    else:
        assert False, f"Unknown LLM fixture: {current_llm}"


class TestSecrets:
    def test_sanitize_secrets(self):
        Secrets.add_secret("mysecret")
        data = "This contains mysecret and ysecre and somemysecreand should be sanitized."
        sanitized = Secrets.sanitize_secrets(data)
        assert sanitized == "This contains ***** and ***** and some*****and should be sanitized."

    @pytest.mark.skipif(
        not os.getenv("RUN_SANITIZATION_TEST"),
        reason="Skipping sensitive tests. Set RUN_SANITIZATION_TEST=1 to run.",
    )
    def test_raise_exception_with_secret(self):
        Secrets.add_secret("mysecret")
        raise Exception("This is a test exception. mysecret exposed!!!")

    def test_sensitive_output_is_sanitized(self):
        # Run pytest for the sensitive tests and capture the output
        result = subprocess.run(
            [
                "pytest",
                "-s",
                "test/test_conftest.py::TestSecrets::test_raise_exception_with_secret",
            ],
            env={**os.environ, "RUN_SANITIZATION_TEST": "1"},
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
        )

        # Combine stdout and stderr to search for secrets
        output = result.stdout + result.stderr

        assert "mysecret" not in output, "Secret exposed in test output!"
        assert "*****" in output, "Sanitization is not working as expected!"
